@@ -11,6 +11,8 @@ import click
 import sys
 import transaction
 
+from libres.db.models import ORMBase
+from libres.context.registry import create_default_registry
 from onegov.core.orm import Base, SessionManager
 from onegov.town.models import Town
 from onegov.town.initial_content import add_initial_content
@@ -24,10 +26,13 @@ def cli(ctx, dsn, schema):
     ctx.obj = {}
 
     mgr = SessionManager(dsn, base=Base)
+    mgr.bases.append(ORMBase)
     mgr.set_current_schema(schema)
 
     ctx.obj['schema'] = schema
+    ctx.obj['session_manager'] = mgr
     ctx.obj['session'] = mgr.session()
+    ctx.obj['libres_registry'] = create_default_registry()
 
 
 @cli.command()
@@ -45,7 +50,12 @@ def add(ctx, name):
         )
         sys.exit(1)
 
-    add_initial_content(session, town_name=name)
+    add_initial_content(
+        ctx.obj['libres_registry'],
+        ctx.obj['session_manager'],
+        town_name=name
+    )
+
     transaction.commit()
 
     click.secho(
