@@ -2,13 +2,18 @@ from cached_property import cached_property
 from onegov.core.layout import ChameleonLayout
 from onegov.core.static import StaticFile
 from onegov.form import FormCollection, FormSubmissionFile, render_field
+from onegov.libres import ResourceCollection
 from onegov.page import Page, PageCollection
 from onegov.people import PersonCollection
 from onegov.ticket import TicketCollection
 from onegov.town import _
 from onegov.town.elements import DeleteLink, Link, LinkGroup
-from onegov.town.models import FileCollection
-from onegov.town.models import ImageCollection, Thumbnail
+from onegov.town.models import (
+    FileCollection,
+    ImageCollection,
+    SiteCollection,
+    Thumbnail
+)
 from sqlalchemy import desc
 
 
@@ -132,6 +137,11 @@ class Layout(ChameleonLayout):
         """ Adds the json url for image lists. """
         return self.request.link(ImageCollection(self.app), name='json')
 
+    @cached_property
+    def sitecollection_url(self):
+        """ Adds the json url for internal links lists. """
+        return self.request.link(SiteCollection(self.app.session()))
+
     @cached_property
     def homepage_url(self):
         """ Returns the url to the main page. """
@@ -160,6 +170,14 @@ class Layout(ChameleonLayout):
             self.request.transform(self.request.path)
         )
 
+    def include_editor(self):
+        self.request.include('redactor')
+        self.request.include('redactor_theme')
+        self.request.include('editor')
+
+    def include_code_editor(self):
+        self.request.include('code_editor')
+
     def thumbnail_url(self, url):
         """ Takes the given url and returns the thumbnail url for it, if it
         exists. Otherwise returns the url as is.
@@ -205,6 +223,7 @@ class DefaultLayout(Layout):
 
         # always include the common js files
         self.request.include('common')
+        self.request.include('common_css')
 
     @cached_property
     def breadcrumbs(self):
@@ -214,7 +233,7 @@ class DefaultLayout(Layout):
     @cached_property
     def root_pages(self):
         query = PageCollection(self.app.session()).query()
-        query = query.order_by(desc(Page.type), Page.name)
+        query = query.order_by(desc(Page.type), Page.title)
         query = query.filter(Page.parent_id == None)
 
         return self.request.exclude_invisible(query.all())
@@ -277,7 +296,7 @@ class AdjacencyListLayout(DefaultLayout):
     def get_sidebar(self, type=None):
         """ Yields the sidebar for the given adjacency list item. """
         query = self.model.siblings.filter(self.model.__class__.type == type)
-        query = query.order_by(self.model.__class__.name)
+        query = query.order_by(self.model.__class__.title)
 
         for item in self.request.exclude_invisible(query.all()):
             url = self.request.link(item)
@@ -289,7 +308,7 @@ class AdjacencyListLayout(DefaultLayout):
 
                 children = sorted(
                     self.request.exclude_invisible(self.model.children),
-                    key=lambda c: c.name
+                    key=lambda c: c.title
                 )
 
                 for item in children:
@@ -326,10 +345,7 @@ class EditorLayout(AdjacencyListLayout):
     def __init__(self, model, request, site_title):
         super(EditorLayout, self).__init__(model, request)
         self.site_title = site_title
-
-        self.request.include('redactor')
-        self.request.include('redactor_theme')
-        self.request.include('editor')
+        self.include_editor()
 
     @cached_property
     def breadcrumbs(self):
@@ -343,11 +359,8 @@ class FormEditorLayout(DefaultLayout):
 
     def __init__(self, model, request):
         super(FormEditorLayout, self).__init__(model, request)
-
-        self.request.include('redactor')
-        self.request.include('redactor_theme')
-        self.request.include('editor')
-        self.request.include('code_editor')
+        self.include_editor()
+        self.include_code_editor()
 
 
 class FormSubmissionLayout(DefaultLayout):
@@ -569,3 +582,134 @@ class TicketLayout(DefaultLayout):
                 ))
 
             return links
+
+
+class ResourcesLayout(DefaultLayout):
+
+    @cached_property
+    def breadcrumbs(self):
+        return [
+            Link(_("Homepage"), self.homepage_url),
+            Link(_("Reservations"), self.request.link(self.model))
+        ]
+
+    @cached_property
+    def editbar_links(self):
+        if self.request.is_logged_in:
+            return [
+                LinkGroup(
+                    title=_("Add"),
+                    links=[
+                        Link(
+                            text=_("Room"),
+                            url=self.request.link(
+                                self.model,
+                                name='neuer-raum'
+                            ),
+                            classes=('new-room', )
+                        ),
+                        Link(
+                            text=_("Daypass"),
+                            url=self.request.link(
+                                self.model,
+                                name='neue-tageskarte'
+                            ),
+                            classes=('new-daypass', )
+                        )
+                    ]
+                ),
+            ]
+
+
+class ResourceLayout(DefaultLayout):
+
+    def __init__(self, model, request):
+        super(ResourceLayout, self).__init__(model, request)
+
+        self.request.include('fullcalendar')
+        self.request.include('fullcalendar_css')
+
+    @cached_property
+    def collection(self):
+        return ResourceCollection(self.request.app.libres_context)
+
+    @cached_property
+    def breadcrumbs(self):
+        return [
+            Link(_("Homepage"), self.homepage_url),
+            Link(_("Reservations"), self.request.link(self.collection)),
+            Link(_(self.model.title), self.request.link(self.model))
+        ]
+
+    @cached_property
+    def editbar_links(self):
+        if self.request.is_logged_in:
+            if self.model.deletable(self.request.app.libres_context):
+                delete_link = DeleteLink(
+                    text=_("Delete"),
+                    url=self.request.link(self.model),
+                    confirm=_("Do you really want to delete this resource?"),
+                    yes_button_text=_("Delete resource"),
+                    redirect_after=self.request.link(self.collection)
+                )
+
+            else:
+                delete_link = DeleteLink(
+                    text=_("Delete"),
+                    url=self.request.link(self.model),
+                    confirm=_("This resource can't be deleted."),
+                    extra_information=_(
+                        "There are existing reservations associated "
+                        "with this resource"
+                    )
+                )
+            return [
+                Link(
+                    text=_("Edit"),
+                    url=self.request.link(self.model, 'bearbeiten'),
+                    classes=('edit-link', )
+                ),
+                delete_link
+            ]
+
+
+class AllocationEditFormLayout(DefaultLayout):
+    """ Same as the resource layout, but with different editbar links, because
+    there's not really an allocation view, but there are allocation forms.
+
+    """
+
+    @cached_property
+    def collection(self):
+        return ResourceCollection(self.request.app.libres_context)
+
+    @cached_property
+    def breadcrumbs(self):
+        return [
+            Link(_("Homepage"), self.homepage_url),
+            Link(_("Reservations"), self.request.link(self.collection)),
+            Link(_("Edit Allocation"), '#')
+        ]
+
+    @cached_property
+    def editbar_links(self):
+        if self.request.is_logged_in:
+            if self.model.availability == 100.0:
+                yield DeleteLink(
+                    _("Delete"),
+                    self.request.link(self.model),
+                    confirm=_("Do you really want to delete this allocation?"),
+                    yes_button_text=_("Delete allocation"),
+                    redirect_after=self.request.link(self.collection)
+                )
+            else:
+                yield DeleteLink(
+                    text=_("Delete"),
+                    url=self.request.link(self.model),
+                    confirm=_("This resource can't be deleted."),
+                    extra_information=_(
+                        "There are existing reservations associated "
+                        "with this resource"
+                    ),
+                    redirect_after=self.request.link(self.collection)
+                )
