@@ -1,21 +1,27 @@
+from onegov.town.models.clipboard import Clipboard
 from onegov.town.models.editor import Editor
 from onegov.town.models.file import File, FileCollection
 from onegov.town.models.form import BuiltinFormDefinition, CustomFormDefinition
 from onegov.town.models.image import Image, ImageCollection, Thumbnail
 from onegov.town.models.page import News, Topic
+from onegov.town.models.resource import DaypassResource
+from onegov.town.models.sitecollection import SiteCollection
 from onegov.town.models.town import Town
 from onegov.town.models.traitinfo import TraitInfo
 
 
 __all__ = [
     'BuiltinFormDefinition',
+    'Clipboard',
     'CustomFormDefinition',
+    'DaypassResource',
     'Editor',
     'File',
     'FileCollection',
     'Image',
     'ImageCollection',
     'News',
+    'SiteCollection',
     'Thumbnail',
     'Topic',
     'Town',
