@@ -2,14 +2,26 @@
 import onegov.core
 import onegov.town
 import textwrap
+import transaction
+import re
 
+from datetime import datetime
+from lxml.html import document_fromstring
 from onegov.form import FormCollection
+from onegov.libres import ResourceCollection
 from onegov.testing import utils
 from onegov.ticket import TicketCollection
 from webtest import TestApp as Client
 from webtest import Upload
 
 
+def extract_href(link):
+    """ Takes a link (<a href...>) and returns the href address. """
+    result = re.search(r'(?:href|ic-delete-from)="([^"]+)', link)
+
+    return result and result.group(1) or None
+
+
 def test_view_permissions():
     utils.assert_explicit_permissions(onegov.town)
 
@@ -189,6 +201,64 @@ def test_login(town_app):
     assert links.text() == 'Login'
 
 
+def test_reset_password(town_app):
+    client = Client(town_app)
+
+    links = client.get('/').pyquery('.bottom-links li:first-child a')
+    assert links.text() == 'Login'
+    login_page = client.get(links.attr('href'))
+
+    request_page = login_page.click(u'Passwort zurücksetzen')
+    assert u'Passwort zurücksetzen' in request_page.text
+
+    request_page.form['email'] = 'someone@example.org'
+    assert 'someone@example.org' in request_page.form.submit().follow()
+    assert len(town_app.smtpserver.outbox) == 0
+
+    request_page.form['email'] = 'admin@example.org'
+    assert 'admin@example.org' in request_page.form.submit().follow()
+    assert len(town_app.smtpserver.outbox) == 1
+
+    message = town_app.smtpserver.outbox[0]
+    message = message.get_payload(0).get_payload(decode=True)
+    message = message.decode('iso-8859-1')
+    link = list(document_fromstring(message).iterlinks())[0][2]
+    token = link.split('token=')[1]
+
+    reset_page = client.get(link)
+    assert token in reset_page.text
+
+    reset_page.form['email'] = 'someone_else@example.org'
+    reset_page.form['password'] = 'new_password'
+    reset_page = reset_page.form.submit()
+    assert u"Ungültiger Addresse oder abgelaufener Link" in reset_page.text
+    assert token in reset_page.text
+
+    reset_page.form['email'] = 'admin@example.org'
+    reset_page.form['password'] = '1234'
+    reset_page = reset_page.form.submit()
+    assert "Feld muss mindestens 8 Zeichen beinhalten" in reset_page.text
+    assert token in reset_page.text
+
+    reset_page.form['email'] = 'admin@example.org'
+    reset_page.form['password'] = 'new_password'
+    assert u"Passwort geändert" in reset_page.form.submit().follow().text
+
+    reset_page.form['email'] = 'admin@example.org'
+    reset_page.form['password'] = 'new_password'
+    reset_page = reset_page.form.submit()
+    assert u"Ungültiger Addresse oder abgelaufener Link" in reset_page.text
+
+    login_page.form['email'] = 'admin@example.org'
+    login_page.form['password'] = 'hunter2'
+    login_page = login_page.form.submit()
+    assert "Unbekannter Benutzername oder falsches Passwort" in login_page.text
+
+    login_page.form['email'] = 'admin@example.org'
+    login_page.form['password'] = 'new_password'
+    assert "Sie wurden eingeloggt" in login_page.form.submit().follow().text
+
+
 def test_settings(town_app):
     client = Client(town_app)
 
@@ -433,8 +503,9 @@ def test_submit_form(town_app):
         E-Mail * = @@@
     """), type='custom')
 
-    client = Client(town_app)
+    transaction.commit()
 
+    client = Client(town_app)
     form_page = client.get('/formulare').click('Profile')
     assert 'Your Details' in form_page
     assert 'First name' in form_page
@@ -477,6 +548,7 @@ def test_pending_submission_error_file_upload(town_app):
         Name * = ___
         Datei * = *.txt|*.csv
     """), type='custom')
+    transaction.commit()
 
     client = Client(town_app)
     form_page = client.get('/formulare').click('Statistics')
@@ -493,6 +565,7 @@ def test_pending_submission_successful_file_upload(town_app):
         Name * = ___
         Datei * = *.txt|*.csv
     """), type='custom')
+    transaction.commit()
 
     client = Client(town_app)
     form_page = client.get('/formulare').click('Statistics')
@@ -596,6 +669,7 @@ def test_show_uploaded_file(town_app):
     collection = FormCollection(town_app.session())
     collection.definitions.add(
         'Text', definition="File * = *.txt\nE-Mail * = @@@", type='custom')
+    transaction.commit()
 
     client = Client(town_app)
 
@@ -872,3 +946,341 @@ def test_tickets(town_app):
 
     assert 'FRM-' in message
     assert '/status' in message
+
+
+def test_resource_slots(town_app):
+
+    resources = ResourceCollection(town_app.libres_context)
+    resource = resources.add("Foo", 'Europe/Zurich')
+
+    scheduler = resource.get_scheduler(town_app.libres_context)
+    scheduler.allocate(
+        dates=[
+            (datetime(2015, 8, 4), datetime(2015, 8, 4)),
+            (datetime(2015, 8, 5), datetime(2015, 8, 5))
+        ],
+        whole_day=True
+    )
+    scheduler.allocate(
+        dates=[
+            (datetime(2015, 8, 6, 12, 0), datetime(2015, 8, 6, 16, 0)),
+        ],
+        whole_day=False
+    )
+    scheduler.approve_reservations(
+        scheduler.reserve(
+            'info@example.org',
+            (datetime(2015, 8, 6, 12, 0), datetime(2015, 8, 6, 16, 0)),
+        )
+    )
+
+    transaction.commit()
+
+    client = Client(town_app)
+
+    url = '/reservation/foo/slots'
+    assert client.get(url).json == []
+
+    url = '/reservation/foo/slots?start=2015-08-04&end=2015-08-05'
+    result = client.get(url).json
+
+    assert len(result) == 2
+
+    assert result[0]['start'] == '2015-08-04T00:00:00+02:00'
+    assert result[0]['end'] == '2015-08-05T00:00:00+02:00'
+    assert result[0]['className'] == 'event-available'
+    assert result[0]['title'] == u"Ganztägig\nVerfügbar"
+
+    assert result[1]['start'] == '2015-08-05T00:00:00+02:00'
+    assert result[1]['end'] == '2015-08-06T00:00:00+02:00'
+    assert result[1]['className'] == 'event-available'
+    assert result[1]['title'] == u"Ganztägig\nVerfügbar"
+
+    url = '/reservation/foo/slots?start=2015-08-06&end=2015-08-06'
+    result = client.get(url).json
+
+    assert len(result) == 1
+    assert result[0]['className'] == 'event-unavailable'
+    assert result[0]['title'] == u"12:00 - 16:00\nBesetzt"
+
+
+def test_resources(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    resources = client.get('/reservationen')
+    assert 'SBB-Tageskarte' in resources
+
+    resource = resources.click('SBB-Tageskarte')
+    assert 'calendar' in resource
+
+    new = resources.click('Raum')
+    new.form['title'] = 'Meeting Room'
+    resource = new.form.submit().follow()
+
+    assert 'calendar' in resource
+    assert 'Meeting Room' in resource
+
+    edit = resource.click('Bearbeiten')
+    edit.form['title'] = 'Besprechungsraum'
+    edit.form.submit()
+
+    assert 'Besprechungsraum' in client.get('/reservationen')
+
+    assert client.delete('/reservation/meeting-room').status_code == 200
+
+
+def test_clipboard(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    page = client.get('/themen/bildung-gesellschaft')
+    assert 'paste-link' not in page
+
+    page = page.click(
+        'Kopieren',
+        extra_environ={'HTTP_REFERER': page.request.url}
+    ).follow()
+
+    assert 'paste-link' in page
+
+    page = page.click('Einf').form.submit().follow()
+    assert '/bildung-gesellschaft/bildung-gesellschaft' in page.request.url
+
+
+def test_clipboard_separation(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    page = client.get('/themen/bildung-gesellschaft')
+    page = page.click('Kopieren')
+
+    assert 'paste-link' in client.get('/themen/bildung-gesellschaft')
+
+    # new client (browser) -> new clipboard
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    assert 'paste-link' not in client.get('/themen/bildung-gesellschaft')
+
+
+def test_copy_pages_to_news(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    page = client.get('/themen/bildung-gesellschaft')
+    edit = page.click('Bearbeiten')
+
+    edit.form['lead'] = '0xdeadbeef'
+    page = edit.form.submit().follow()
+
+    page.click('Kopieren')
+
+    edit = client.get('/aktuelles').click('Einf')
+
+    assert '0xdeadbeef' in edit
+    page = edit.form.submit().follow()
+
+    assert '/aktuelles/bildung-gesellschaft' in page.request.url
+
+
+def test_sitecollection(town_app):
+    client = Client(town_app)
+
+    assert client.get('/sitecollection', expect_errors=True).status_code == 403
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    collection = client.get('/sitecollection').json
+
+    assert collection[0] == {
+        'name': 'Bildung & Gesellschaft',
+        'url': 'http://localhost/themen/bildung-gesellschaft',
+        'group': 'Themen'
+    }
+
+
+def test_allocations(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    # create a new daypass allocation
+    new = client.get((
+        '/reservation/sbb-tageskarte/neue-einteilung'
+        '?start=2015-08-04&end=2015-08-05'
+    ))
+
+    new.form['daypasses'] = 1
+    new.form['daypasses_limit'] = 1
+    new.form.submit()
+
+    # view the daypasses
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-04&end=2015-08-05'
+    ))
+
+    assert len(slots.json) == 2
+    assert slots.json[0]['title'] == u"Ganztägig\nVerfügbar"
+
+    # change the daypasses
+    edit = client.get(extract_href(slots.json[0]['actions'][1]))
+    edit.form['daypasses'] = 2
+    edit.form.submit()
+
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-04&end=2015-08-04'
+    ))
+
+    assert len(slots.json) == 1
+    assert slots.json[0]['title'] == u"Ganztägig\n2/2 Verfügbar"
+
+    # try to create a new allocation over an existing one
+    new = client.get((
+        '/reservation/sbb-tageskarte/neue-einteilung'
+        '?start=2015-08-04&end=2015-08-04'
+    ))
+
+    new.form['daypasses'] = 1
+    new.form['daypasses_limit'] = 1
+    new = new.form.submit()
+
+    assert u"Es besteht bereits eine Einteilung im gewünschten Zeitraum" in new
+
+    # move the existing allocations
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-04&end=2015-08-05'
+    ))
+
+    edit = client.get(extract_href(slots.json[0]['actions'][1]))
+    edit.form['date'] = '2015-08-06'
+    edit.form.submit()
+
+    edit = client.get(extract_href(slots.json[1]['actions'][1]))
+    edit.form['date'] = '2015-08-07'
+    edit.form.submit()
+
+    # get the new slots
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-06&end=2015-08-07'
+    ))
+
+    assert len(slots.json) == 2
+
+    # delete an allocation
+    client.delete(extract_href(slots.json[0]['actions'][2]))
+
+    # get the new slots
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-06&end=2015-08-07'
+    ))
+
+    assert len(slots.json) == 1
+
+    # delete an allocation
+    client.delete(extract_href(slots.json[0]['actions'][2]))
+
+    # get the new slots
+    slots = client.get((
+        '/reservation/sbb-tageskarte/slots'
+        '?start=2015-08-06&end=2015-08-07'
+    ))
+
+    assert len(slots.json) == 0
+
+
+def test_allocation_times(town_app):
+    client = Client(town_app)
+
+    login_page = client.get('/login')
+    login_page.form.set('email', 'admin@example.org')
+    login_page.form.set('password', 'hunter2')
+    login_page.form.submit()
+
+    new = client.get('/reservationen').click('Raum')
+    new.form['title'] = 'Meeting Room'
+    new.form.submit()
+
+    # 12:00 - 24:00
+    new = client.get('/reservation/meeting-room/neue-einteilung')
+    new.form['start'] = '2015-08-20'
+    new.form['end'] = '2015-08-20'
+    new.form['start_time'] = '12:00'
+    new.form['end_time'] = '24:00'
+    new.form['as_whole_day'] = 'no'
+    new.form.submit()
+
+    slots = client.get(
+        '/reservation/meeting-room/slots?start=2015-08-20&end=2015-08-20'
+    )
+
+    assert len(slots.json) == 1
+    assert slots.json[0]['start'] == '2015-08-20T12:00:00+02:00'
+    assert slots.json[0]['end'] == '2015-08-21T00:00:00+02:00'
+
+    # 00:00 - 02:00
+    new = client.get('/reservation/meeting-room/neue-einteilung')
+    new.form['start'] = '2015-08-22'
+    new.form['end'] = '2015-08-22'
+    new.form['start_time'] = '00:00'
+    new.form['end_time'] = '02:00'
+    new.form['as_whole_day'] = 'no'
+    new.form.submit()
+
+    slots = client.get(
+        '/reservation/meeting-room/slots?start=2015-08-22&end=2015-08-22'
+    )
+
+    assert len(slots.json) == 1
+    assert slots.json[0]['start'] == '2015-08-22T00:00:00+02:00'
+    assert slots.json[0]['end'] == '2015-08-22T02:00:00+02:00'
+
+    # 12:00 - 24:00 over two days
+    new = client.get('/reservation/meeting-room/neue-einteilung')
+    new.form['start'] = '2015-08-24'
+    new.form['end'] = '2015-08-25'
+    new.form['start_time'] = '12:00'
+    new.form['end_time'] = '24:00'
+    new.form['as_whole_day'] = 'no'
+    new.form.submit()
+
+    slots = client.get(
+        '/reservation/meeting-room/slots?start=2015-08-24&end=2015-08-25'
+    )
+
+    assert len(slots.json) == 2
+    assert slots.json[0]['start'] == '2015-08-24T12:00:00+02:00'
+    assert slots.json[0]['end'] == '2015-08-25T00:00:00+02:00'
+    assert slots.json[1]['start'] == '2015-08-25T12:00:00+02:00'
+    assert slots.json[1]['end'] == '2015-08-26T00:00:00+02:00'
